# Sketchup To Ogre Exporter  Version 1.0.1
# Written by Kojack
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA, or go to
# http://www.gnu.org/copyleft/lesser.txt.



def remove_spaces(s)
	for i in (0..s.length)
		if s[i,1] == " " 
			s[i] = "_" 
		end 
	end
	s
end

def append_paths(p,f)
	if p[-1,1] == "\\" or p[-1,1] == "/"
		p+f
	else
		p+"\\"+f
	end
end


def ogre_export_entities(out, entity_list, trans, mat)
	for e in entity_list
		if e.class == Sketchup::Group
			if e.material != nil
				mat = e.material
			end
			ogre_export_entities(out, e.entities, trans*e.transformation, mat)
		end
		if e.class == Sketchup::ComponentInstance
			if e.material != nil
				mat = e.material
			end
			ogre_export_entities(out, e.definition.entities, trans*e.transformation, mat)
		end
		if e.class == Sketchup::Face
			if e.material == nil and mat != nil
			end
			if e.material != nil
				mat = e.material
			end
			if $g_ogre_materials[mat] == nil
				$g_ogre_materials[mat] = []
			end
			$g_ogre_materials[mat] = $g_ogre_materials[mat] + [[e,trans,mat]]
			$g_ogre_texturewriter.load(e,true)
		end
	end
end

def point_to_vector(p)
	Geom::Vector3d.new(p.x,p.y,p.z)
end

def ogre_export_face(out,mat)
	meshes = []
	polycount = 0
	pointcount = 0
	mirrored={}
	for ft in ($g_ogre_materials[mat])
		polymesh = ft[0].mesh 7
		polymesh.transform! ft[1]
		mirrored[polymesh] = true
		xa = point_to_vector(ft[1].xaxis)
		ya = point_to_vector(ft[1].yaxis)
		za = point_to_vector(ft[1].zaxis)
		xy = xa.cross(ya)
		xz = xa.cross(za)
		yz = ya.cross(za)
		if xy.dot(za) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if xz.dot(ya) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if yz.dot(xa) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		meshes = meshes + [[polymesh,ft[0].material]]
		
		polycount=polycount + polymesh.count_polygons
		pointcount=pointcount + polymesh.count_points
	end
	startindex = 0
	has_texture = false
	if mat
		matname = remove_spaces(mat.display_name)
		if mat.texture
			has_texture = true
		end
	else
		matname = "SketchupDefault"
	end
	out.print "      <submesh material = \"#{matname}\" usesharedvertices=\"false\" "
	if pointcount<65537 
		out.print "use32bitindexes=\"false\">\n"
	else
		out.print "use32bitindexes=\"true\">\n"
	end
	$g_ogre_count_submesh = $g_ogre_count_submesh + 1
	out.print "         <faces count=\"#{polycount}\">\n"
	for mesh in meshes
		for poly in mesh[0].polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+startindex
			v2 = (poly[1]>=0?poly[1]:-poly[1])+startindex
			v3 = (poly[2]>=0?poly[2]:-poly[2])+startindex
			if !mirrored[mesh[0]]
				out.print "            <face v1=\"#{v1-1}\" v2=\"#{v2-1}\" v3=\"#{v3-1}\" />\n"
			else
				out.print "            <face v1=\"#{v2-1}\" v2=\"#{v1-1}\" v3=\"#{v3-1}\" />\n"
			end
			$g_ogre_count_tri = $g_ogre_count_tri + 1
		end
		startindex = startindex + mesh[0].count_points
	end
	out.print "         </faces>\n"
	out.print "         <geometry vertexcount=\"#{pointcount}\">\n"
	out.print "            <vertexbuffer positions=\"true\" normals=\"true\" colours_diffuse=\"false\" "
	if has_texture 
		out.print "texture_coords=\"1\" texture_coord_dimensions_0=\"2\""
	end
	out.print " >\n"
	
	for mesh in meshes
		$g_test = mesh
		for p in (1..mesh[0].count_points)
			pos = mesh[0].point_at(p).to_a
			norm = mesh[0].normal_at(p).to_a
			if has_texture and mesh[1] == nil
				texsize = Geom::Point3d.new(mat.texture.width, mat.texture.height, 1)
			else
				texsize = Geom::Point3d.new(1,1,1)
			end
#			uv = (mesh[0].uv_at(p,1)/texsize).to_a
			uv = [mesh[0].uv_at(p,1).x/texsize.x, mesh[0].uv_at(p,1).y/texsize.y, mesh[0].uv_at(p,1).z/texsize.z]
			out.print "               <vertex>\n"
			out.print "                  <position x=\"#{pos[0]*$g_ogre_scale}\" y=\"#{pos[2]*$g_ogre_scale}\" z=\"#{pos[1]*-$g_ogre_scale}\" />\n"
			out.print "                  <normal x=\"#{norm[0]}\" y=\"#{norm[2]}\" z=\"#{-norm[1]}\" />\n"
			if has_texture 
				out.print "                  <texcoord u=\"#{uv[0]}\" v=\"#{-uv[1]+1}\" />\n"
			end
			out.print "               </vertex>\n"
		end
	end
	out.print "            </vertexbuffer>\n"
	out.print "         </geometry>\n"
	out.print "      </submesh>\n"
end

def ogre_export_materials(out, mats)
    for m in mats
		out.print "material #{remove_spaces(m.display_name)}\n"
		out.print "{\n"
		out.print "   technique\n"
		out.print "   {\n"
		out.print "      pass\n"
		out.print "      {\n"
		if m.use_alpha?
			out.print "         scene_blend alpha_blend\n"
			out.print "         depth_check on\n"
			out.print "         depth_write off\n"
		end
		if m.texture
			out.print "         #{m.texture.width}, #{m.texture.height}\n"
			filename = ""
			m.texture.filename.each("\\") {|n| filename = n}
			out.print "         diffuse 1 1 1 #{m.alpha}\n"
			out.print "         texture_unit\n"
			out.print "         {\n"
			out.print "            texture #{filename}\n"
			out.print "         }\n"
		else
			out.print "         diffuse #{m.color.red/255.0} #{m.color.green/255.0} #{m.color.blue/255.0} #{m.alpha}\n"
		end
		out.print "      }\n"
		out.print "   }\n"
		out.print "}\n"
		$g_ogre_count_material = $g_ogre_count_material + 1
	end
	out.print "material SketchupDefault\n"
	out.print "{\n"
	out.print "   technique\n"
	out.print "   {\n"
	out.print "      pass\n"
	out.print "      {\n"
	colour = Sketchup.active_model.rendering_options["FaceFrontColor"]
	out.print "         diffuse #{colour.red/255.0} #{colour.green/255.0} #{colour.blue/255.0}\n"
	out.print "      }\n"
	out.print "   }\n"
	out.print "}\n"
	$g_ogre_count_material = $g_ogre_count_material + 1
end

def ogre_export
	ogre_export_main false
end

def ogre_export_selected
	ogre_export_main true
end

def ogre_export_main(selected)
	$g_ogre_scale = 0.0254
	$g_ogre_copy_textures = true
	$g_ogre_path_meshes = "c:\\"
	$g_ogre_path_materials = "c:\\"
	$g_ogre_path_textures = "c:\\"
	$g_ogre_path_xml_converter = "c:\\OgreXmlConverter.exe"
	$g_ogre_export_materials = true
	$g_ogre_export_meshes = true
	$g_ogre_convert_xml = true
	$g_ogre_delete_xml = false
	load"ogre_config.rb"
	$g_ogre_count_tri = 0
	$g_ogre_count_submesh = 0
	$g_ogre_count_material = 0
	$g_ogre_faces = []
	$g_ogre_trans = []
	$g_ogre_materials = {}
	$g_ogre_texturewriter = Sketchup.create_texture_writer

	temp_name = (UI.inputbox ["Export Name"],[$g_ogre_export_name],"Export To Ogre")
	if temp_name == false
		return
	end
	$g_ogre_export_name = temp_name[0]
	if $g_ogre_export_name == ""
		return
	end
		
	if $g_ogre_export_meshes == true
		file_mesh_xml = open(append_paths($g_ogre_path_meshes,$g_ogre_export_name+".mesh.xml"),"w")
		file_mesh_xml.print "<mesh>\n"
		file_mesh_xml.print "   <submeshes>\n"
		if selected
			ents = []
			Sketchup.active_model.selection.each{|e| ents = ents + [e]}
			ogre_export_entities(file_mesh_xml, ents, Geom::Transformation.new, nil)
		else
			ogre_export_entities(file_mesh_xml, Sketchup.active_model.entities , Geom::Transformation.new, nil)
		end
		for mat in Sketchup.active_model.materials
			if $g_ogre_materials[mat]
				ogre_export_face(file_mesh_xml,mat)
			end
		end
		if $g_ogre_materials[nil]
			ogre_export_face(file_mesh_xml,nil)
		end
		file_mesh_xml.print "   </submeshes>\n"
		file_mesh_xml.print "</mesh>\n"
		file_mesh_xml.close
	end

	if $g_ogre_export_materials == true
		file_material = open(append_paths($g_ogre_path_materials,$g_ogre_export_name+".material"),"w")
		ogre_export_materials(file_material, Sketchup.active_model.materials)
		file_material.close
	end
	UI.messagebox "Triangles = #{$g_ogre_count_tri}\nSubmeshes = #{$g_ogre_count_submesh}\nMaterials = #{$g_ogre_count_material}"
	if $g_ogre_convert_xml == true
		system($g_ogre_path_xml_converter + " " + append_paths($g_ogre_path_meshes,$g_ogre_export_name+".mesh.xml"))
	end
	if $g_ogre_copy_textures == true
		$g_ogre_texturewriter.write_all $g_ogre_path_textures,false
	end
	$g_ogre_texturewriter=nil
end

$g_ogre_export_name = ""

menu = UI.menu "Tools";
menu.add_separator
menu.add_item( "Export To Ogre") {ogre_export}
menu.add_item( "Export Selected To Ogre") {ogre_export_selected}
